{% set invenio_request = notification.context.request %}
{% set invenio_request_event = notification.context.request_event %}
{% set invenio_topic = invenio_request.topic or {} %}

{# created_by is either a resolved user or an email (for guests) #}
{% set event_creator_name = (invenio_request_event.created_by.profile and invenio_request_event.created_by.profile.full_name) or invenio_request_event.created_by.username or invenio_request_event.created_by.email or invenio_request_event.created_by %}
{% set request_id = invenio_request.id %}
{% set request_event_content = invenio_request_event.payload.content | safe %}
{% set request_title = invenio_request.title | safe %}
{% set topic_title = ( invenio_topic.title or invenio_topic.metadata.title ) | safe %}

{# TODO: use request.links.self_html when issue issue is resolved: https://github.com/inveniosoftware/invenio-rdm-records/issues/1327 #}
{% set request_link = "{ui}/me/requests/{id}".format(
    ui=config.SITE_UI_URL, id=request_id
    )
%}
{% set account_settings_link = "{ui}/account/settings/notifications".format(
    ui=config.SITE_UI_URL
    )
%}

{%- block subject -%}
{%- if topic_title -%}
{{ _("💬 New comment on on '{request_title}' for '{topic_title}'").format(request_title=request_title, topic_title=topic_title) }}
{%- elif request_title -%}
{{ _("💬 New comment on on '{request_title}'").format(request_title=request_title) }}
{%- else -%}
{{ _("💬 New comment") }}
{%- endif -%}
{%- endblock subject -%}

{%- block html_body -%}
<table style="font-family:'Lato',Helvetica,Arial,sans-serif;border-spacing:15px">
    <tr>
        {% if topic_title %}
        <td>{{ _("'{user_name}' commented on '{request_title}' for '{topic_title}':").format(user_name=event_creator_name, request_title=request_title, topic_title=topic_title) }}</td>
        {% elif request_title %}
        <td>{{ _("'{user_name}' commented on '{request_title}':").format(user_name=event_creator_name, request_title=request_title) }}</td>
        {% else %}
        <td>{{ _("'{user_name}' commented:").format(user_name=event_creator_name) }}</td>
        {% endif %}
    </tr>
    <tr>
        <td><em>{{ request_event_content }}</em></td>
    </tr>
    <tr>
        <td><a href="{{ request_link }}" class="button">{{ _("Check out the request")}}</a></td>
    </tr>
    <tr>
        <td><strong>_</strong></td>
    </tr>
    <tr>
        <td style="font-size:smaller">{{ _("This is an auto-generated message. To manage notifications, visit your")}} <a href="{{account_settings_link}}">{{ _("account settings")}}</a>.</td>
    </tr>
</table>
{%- endblock html_body %}

{%- block plain_body -%}
{{ _("@{user_name} commented on '{request_title}'").format(user_name=event_creator_name, request_title=request_title) }}.

{{ request_event_content }}

{{ _("Check out the request: {request_link}").format(request_link=request_link) }}

{%- endblock plain_body %}

{# Markdown for Slack/Mattermost/chat #}
{%- block md_body -%}
{{ _("*@{user_name}* commented on *{request_title}*").format(user_name=event_creator_name, request_title=request_title) }}.

{{ request_event_content }}

[{{_("Check out the request")}}]({{request_link}})
{%- endblock md_body %}
