{% set access_request = notification.context.request %}
{% set topic = notification.context.request.topic %}
{% set request_html_page = notification.context.request.links.self_html %}
{% set topic_title = notification.context.request.topic.get("metadata", {}).get("title", topic.id) %}

{%- block subject -%}
    {{ _("❗️ Published record has been deleted") }}
{%- endblock subject -%}

{%- block html_body -%}
    {{ _("The published record has been deleted.") }}
    {{ _("For more information head to the <a href='%(link)s'>request page</a>.", link=request_html_page)}}
    <table style="font-family:'Lato',Helvetica,Arial,sans-serif;border-spacing:15px">
        <tr>
            <td style="font-size:smaller">{{ _("This is an auto-generated message. To manage notifications, visit your")}} <a href="{{account_settings_link}}">{{ _("account settings")}}</a>.</td>
        </tr>
    </table>
{%- endblock html_body -%}

{%- block plain_body -%}
{{ _("The published record has been deleted.") }}
{{ _("For more information head to the %(link)s", link=request_html_page)}} 

{{ _("This is an auto-generated message. To manage notifications, visit your account settings") }}.
{%- endblock plain_body -%}
