# noqa INP001

#
# Copyright (C) 2024 CESNET z.s.p.o.
#
# oarepo-requests is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.
#
from __future__ import annotations

from invenio_i18n import _

_("Create Request")
_("Open dialog for request")
_("My Requests")
_("Requests to Approve")
_("Are you sure?")
_("Cancel")
_("OK")
_("Create request")
_("Submit request")
_("Delete request")
_("Delete")
_("Accept request")
_("Accept")
_("Decline request")
_("Decline")
_("Create and submit request")
_("Create and submit")
_("Error sending request")
_("Submit")
_("Save drafted request")
_("Save")
_("Create")
_("Creator")
_("Receiver")
_("Request type")
_("Created")
_("Timeline")
_("Submit event")
_("No requests to show")
_("api.requests")
_("api.applicable-requests")
